%% MEFA_SDA : Exploratory Factor Analysis for Multiple imputated datasets when controlling response bias
%
% ARGUMENTS   : Xm   -> data with imputate values (N x m x k)
%               r    -> number of factors to retain
%               corr -> Correlation matrix to be used
%                           1: Variance
%                           2: Pearson
%                           3: Tetrachoric/polychoric correlation
%               rot  -> Rotation to be used
%                           %ORIGINAL               %THIS MODIFIED VERSION:
%                           0: No rotate            0: No rotate
%                           1: Consensus varimax    1: Consensus varimax
%                           2: Consensus oblimin    2: Consensus Promin
%                           3: Consensus Promin
%               G    -> Vector containing the number of SD items and the number of content items
%               iba  -> Balanced set of items
%               iuba -> Items not included in the balanced core ("Unbalanced set of items")
%               checking_DS -> Logic value specifying if Social Desirability has to be controled
%               checking_AQ -> Logic value specifying if Acquiescence has to be controled
%               P_sensemiss -> P matrix computed previously only with the non missing data
%               sco  -> Compute factor scores:
%                           0: Don't compute (faster)
%                           1: Compute factor scores (slower, specially using polychoric correlation) 
%               H    -> Theoretical target matrix (optional)
%
% RETURN VALUE: P    -> Consensus pattern matrix (m x r)
%               PHI  -> Consensus inter-factor correlation matrix (r x r)
%               F    -> Consensus factor scores (N x r)
%               Fint -> Consensus factor scores: minimum and maximum score (N x 2 x r)
%               F_li -> Lower limit of F
%               F_ls -> Upper limit of F
%               Se   -> Standard Errors
%               reli -> Reliabilities
%
% DEPENDENCIES: list of 45 functions, for viewing all type:
%                   matlab.codetools.requiredFilesAndProducts('mefa_sda.m')
%

function [P,PHI,F,Fint,F_li,F_ls,Se,reli]=mefa_sda(Xm,r,corr,rot,G,iba,iuba,checking_DS,checking_AQ,P_sensemiss,sco,H)

[N,m,k]=size(Xm);

conv = 0.00001;

mc=0;
for ik=1:k,
    if max(max(Xm(:,:,ik))') > mc,
        mc = max(max(Xm(:,:,ik))');
    end;
end;

Rm=zeros(m,m,k); Pm=zeros(m,r+checking_DS+checking_AQ,k); THRESm=zeros(mc,m,k);
P_biaxos=zeros(m,checking_DS+checking_AQ,k);
Pc=zeros(G(2),r,k);
if corr==3
    if m<=30
        estimated_time='less than 5 minutes';
    elseif m>30 && m<=50
        estimated_time='between 5 and 10 minutes';
    elseif m>50 && m<=80
        estimated_time='between 10 and 20 minutes';
    elseif m>80 && m<=120
        estimated_time='between 20 and 40 minutes';
    elseif m>=120
        estimated_time='more than an hour';
    end;
    estimation={};
    estimation{1}=sprintf('Computing MEFA ... Estimated time remaining = %s',estimated_time);
    estimation{2}='(Highly dependent on the CPU speed)';
    BarCount = 0; bar1 = waitbar(BarCount,estimation);
    pos_1=get(bar1,'position');
    pos_2=[pos_1(1) pos_1(2)+(1.5*pos_1(4)) pos_1(3) pos_1(4)];
    set(bar1,'position',pos_2,'doublebuffer','on');
else
    BarCount = 0; bar1 = waitbar(BarCount,'Computing MEFA ... Please wait');
end;
TicPrint = tic;
TotalLines=k;
Pc_2=[];
rm=ones(1,k)*r;
for ik=1:k,
    %% correlation matrix
    switch corr
        case 1,     %Variance
            Ri=cov(Xm(:,:,ik));
        case 2,     %Pearson
            Ri=corrcoef(Xm(:,:,ik));
        case 3,     %Polychoric
            [Ri,THRESi]=polychomap(Xm(:,:,ik),min(Xm(:,:,ik)),max(Xm(:,:,ik)),3,0,1);
            THRESm(:,:,ik) = [THRESi'; zeros(1,m)];
        otherwise,
            Ri=corrcoef(Xm(:,:,ik));
    end;
    Rm(:,:,ik)=Ri;
    
    tmp=sprintf('Correlation matrix %.0f done!',ik);disp(tmp);
   
    % Control DS/AQ
    if checking_DS==1 && checking_AQ==1                
        [P,EV,GFI,AGFI,RMSR,kelly]=control_sda_r(Ri,N,r,G,iba,iuba);
    elseif checking_DS==1 && checking_AQ==0
        [P,EV,GFI,AGFI,RMSR,kelly]=control_sd_r(Ri,N,r,G,iba,iuba);
    elseif checking_DS==0 && checking_AQ==1
        [P,EV,GFI,AGFI,RMSR,kelly]=control_a_r(Ri,N,r,G,iba,iuba);
    end;
    
    Pm(:,:,ik) = P;
    P_biaxos(:,:,ik) = Pm(:,(1:checking_DS+checking_AQ),ik);
    Pc(:,:,ik) = Pm((G(1)+1):m,(checking_DS+checking_AQ+1):r+checking_DS+checking_AQ,ik);
    Pc_2 = [Pc_2 Pc(:,:,ik)];

    tmp=sprintf('Loading matrix %.0f done!',ik);disp(tmp);
    BarCount = BarCount + 1; 
    compT = toc(TicPrint);
    compT = compT*(TotalLines-BarCount)/BarCount;
    if ik==k
        s_tmp={};
        s_tmp{1}=sprintf('Computing MEFA ... Actual imputation =  %.0f  / %.0f ) ',ik,k);
        s_tmp{2}=sprintf('(Remaining time =  %s )',Time2String( compT ));
        waitbar(BarCount/TotalLines,bar1,s_tmp);
        
    else
        s_tmp={};
        s_tmp{1}=sprintf('Computing MEFA ... Actual imputation =  %.0f  / %.0f ) ',ik+1,k);
        s_tmp{2}=sprintf('(Remaining time =  %s )',Time2String( compT ));
        waitbar(BarCount/TotalLines,bar1,s_tmp);
    end;
    
    %Rotation for each imputation %%% NOMES PER AL CALCUL DE LES
    %PUNTUACIONS FACTORIALS DE CADA SUBJECTE, NO TE EFECTE EN LA ROTACIO DE
    %CONSENS
    switch rot
        case 1 %orthogonal
            Tco = varim(Pc(:,:,ik),conv,10);
            Pc(:,:,ik)=Pc(:,:,ik)*Tco;

        case 2 %Oblique
            [Tco,~,~] = promin(Pc(:,:,ik),conv,10);
            Pc(:,:,ik) = Pc(:,:,ik)*Tco;

        case 3 % target?? orrown?
            [Pc(:,:,ik), PHIr] = tarrot_obl (Pc(:,:,ik), abs(H), 0.001);
            [Pc(:,:,ik),~]=orrown(H,Pc(:,:,ik),PHIr);
            
        case 4 % target?? orrown?
            PHIr=eye(r);
            Pc(:,:,ik) = tarrotorth (Pc(:,:,ik), abs(H), 0.001);
            Pc(:,:,ik)=orrown(H,Pc(:,:,ik),PHIr);
        

    end;
    
end;

%Consensus rotation
rm=ones(1,k)*r;
PHI = eye(r+checking_DS+checking_AQ);
switch rot
    case 1 %orthogonal
        PHIt=eye(r);
        [Tco,P] = cvarim(Pc_2,k,rm,conv,10);
        
    case 2 %Oblique
        [Tco,PHIt,P] = cpromin(Pc_2,k,rm,conv,10);
        
    otherwise %I la rotacio target?? orrown?
        PHIt=eye(r);
        [Tco,P] = cunrotated(Pc_2,k,rm,conv,10);
        
end;

close(bar1);
disp('Consensus rotation done!');

%ajustarem a la P sense missings
[P,PHIt] = orrown(P_sensemiss,P,PHIt);

P_biaxos2 = zeros(m, checking_DS+checking_AQ);
for ik=1:k,
    P_biaxos2 = P_biaxos2 + P_biaxos(:,:,ik);
end;

P_biaxos_promig = P_biaxos2 / k;

if nargin==12, %si hi ha target %WORK IN PROGRESS
    switch rot
        case 3
            [Pr, PHIr] = tarrot_obl (P, abs(H), 0.001);
            [P,PHIt]=orrown(H,Pr,PHIr);
            P=[zeros(G(1),r);P]; 
            P=[P_biaxos_promig P];
        case 4
            PHIr=eye(r);
            Pr = tarrotorth (P, abs(H), 0.001);
            P=orrown(H,Pr,PHIr);
            PHIt=eye(r);
            P=[zeros(G(1),r);P]; 
            P=[P_biaxos_promig P];
    end;
else
    P=[zeros(G(1),r);P];
    P=[P_biaxos_promig P];  
end;

PHI((checking_DS+checking_AQ+1:r+checking_DS+checking_AQ),(checking_DS+checking_AQ+1:r+checking_DS+checking_AQ)) = PHIt;
for ik=1:k
    [Pc_ajustat(:,:,ik),PHIt] = orrown(P_sensemiss,Pc(:,:,ik),PHIt);
    buff=[zeros(G(1),r);Pc_ajustat(:,:,ik)];
    Pm_ajustat(:,:,ik)=[P_biaxos(:,:,ik) buff];
end;

% FACTOR SCORES FOR EACH SAMPLE 
Fm=zeros(N,r+checking_DS+checking_AQ,k);
Fm_li=zeros(N,r+checking_DS+checking_AQ,k);
Fm_ls=zeros(N,r+checking_DS+checking_AQ,k);
if corr==3
    Sem=zeros(N,r+checking_DS+checking_AQ,k);
    relim=zeros(N,r+checking_DS+checking_AQ,k);
    Se=zeros(N,r+checking_DS+checking_AQ);
    reli=zeros(N,r+checking_DS+checking_AQ);
else
    Sem=zeros(r+checking_DS+checking_AQ,r+checking_DS+checking_AQ,k);
    relim=zeros(r+checking_DS+checking_AQ,1,k);
    Se=zeros(r+checking_DS+checking_AQ,r+checking_DS+checking_AQ);
    reli=zeros(r+checking_DS+checking_AQ,1);
end;
F=zeros(N,r+checking_DS+checking_AQ);
F_li=zeros(N,r+checking_DS+checking_AQ);
F_ls=zeros(N,r+checking_DS+checking_AQ);
Fint=zeros(N,2,r+checking_DS+checking_AQ);

%%%% FACTOR SCORES AS AVERAGE
% save tmp
if sco==1,
    if corr==3 %GLOBAL WAITBAR FOR COMPUTING FACTOR SCORES
        tmp={};
        tmp{1}=('Computing factor scores, this operation can take a while, depending of');
        tmp{2}=sprintf('the number of items and imputations (5 by default) ... (Actual imputation = 1 / %.0f  )',k);
        BarCount = 0;
        bar0 = waitbar(BarCount,tmp);
        pos_1=get(bar0,'position');
        pos_2=[pos_1(1) pos_1(2)+(1.5*pos_1(4)) pos_1(3) pos_1(4)];
        set(bar0,'position',pos_2,'doublebuffer','on');
        TicPrint = tic;
        TotalLines=k;
    end;
    
    for ik=1:k,
    %     Pi = Pm(:,:,ik)*part(Tco,ik,r); %no es pot fer, perque el Tco el
    %     treiem de fer la rotacio sobre els factors de contingut
        switch corr
            case 1,     %Variance
                [Fi,Fi_li,Fi_ls,Se2,reli2] = eap_continuous_obli(Xm(:,:,ik), Pm_ajustat(:,:,ik), PHI,Rm(:,:,ik));
            case 2,     %Pearson
                [Fi,Fi_li,Fi_ls,Se2,reli2] = eap_continuous_obli(Xm(:,:,ik), Pm_ajustat(:,:,ik), PHI,Rm(:,:,ik));
            case 3,     %Polychoric
                
                %check if graded model is selected
                data=getappdata(0,'data');
                
                if data.graded==1
                     n_nodes=data.n_nodes;
                
                     Bi=THRESm(:,:,ik);
                     sigj_i = sqrt(ones(m,1)-diag(Pm_ajustat(:,:,ik)*PHI*Pm_ajustat(:,:,ik)'));
                     [Fi,Fi_li,Fi_ls,Se2,reli2]  = reap_grad_obli(Xm(:,:,ik), Pm_ajustat(:,:,ik), PHI, Bi, sigj_i,n_nodes,1); 
                     
                else 
                    %lineal model using the polychoric correlations
                    [Fi,Fi_li,Fi_ls,Se2,reli2] = eap_continuous_obli(Xm(:,:,ik), Pm_ajustat(:,:,ik), PHI,Rm(:,:,ik));
                 
                end
                 %it has to be done in separate steps:
%                 m=G(1)+G(2);
%                 [g1,g2]=size(P);
%                 if checking_DS==1
%                     [Fi_a,Fi_li_a,Fi_ls_a,Se2_a,reli2_a] = reap_grad_obli(Xm(:,:,ik), Pm_ajustat(:,1,ik), 1, Bi, sigj_i,1,1);
%                     
%                     if checking_AQ==1
%                         %AC
%                         [Fi_b,Fi_li_b,Fi_ls_b,Se2_b,reli2_b] = reap_grad_obli(Xm(:,G(1)+1:m,ik), Pm_ajustat(G(1)+1:m,2,ik), 1, Bi(:,G(1)+1:m), sigj_i(G(1)+1:m),1,2);
%                         
%                         %content
%                         [Fi_c,Fi_li_c,Fi_ls_c,Se2_c,reli2_c] = reap_grad_obli(Xm(:,G(1)+1:m,ik), Pm_ajustat(G(1)+1:m,3:g2,ik), PHIt, Bi(:,G(1)+1:m), sigj_i(G(1)+1:m),1,3);
%                     else
%                         %AC (NO)
%                         Fi_b=[]; Fi_li_b=[]; Fi_ls_b=[]; Se2_b=[]; reli2_b=[];
%                         
%                         %content
%                         [Fi_c,Fi_li_c,Fi_ls_c,Se2_c,reli2_c] = reap_grad_obli(Xm(:,G(1)+1:m,ik), Pm_ajustat(G(1)+1:m,2:g2,ik), PHIt, Bi(:,G(1)+1:m), sigj_i(G(1)+1:m),1,3);
%                     end;
%                 else
%                     %SD (no)
%                     Fi_a=[]; Fi_li_a=[]; Fi_ls_a=[]; Se2_a=[]; reli2_a=[];
%                     if checking_AQ==1
%                         [Fi_b,Fi_li_b,Fi_ls_b,Se2_b,reli2_b] = reap_grad_obli(Xm(:,:,ik), Pm_ajustat(:,1,ik), 1, Bi, sigj_i,1,2);
%                         
%                         %content
%                         [Fi_c,Fi_li_c,Fi_ls_c,Se2_c,reli2_c] = reap_grad_obli(Xm(:,:,ik), Pm_ajustat(:,2:g2,ik), PHIt, Bi, sigj_i,1,3);
%                     else
%                         %AC (NO)
%                         Fi_b=[]; Fi_li_b=[]; Fi_ls_b=[]; Se2_b=[]; reli2_b=[];
%                         
%                         %content
%                         [Fi_c,Fi_li_c,Fi_ls_c,Se2_c,reli2_c] = reap_grad_obli(Xm(:,:,ik), Pm_ajustat(:,:,ik), PHIt, Bi, sigj_i,1,3);
%                     end;
%                     
%                 end;
%                 Fi=[Fi_a Fi_b Fi_c];
%                 Fi_li=[Fi_li_a Fi_li_b Fi_li_c];
%                 Fi_ls=[Fi_ls_a Fi_ls_b Fi_ls_c];
%                 Se2=[Se2_a Se2_b Se2_c];
%                 reli2=[reli2_a reli2_b reli2_c];
                 
            otherwise,
                [Fi,Fi_li,Fi_ls,Se2,reli2] = eap_continuous_obli(Xm(:,:,ik), Pm_ajustat(:,:,ik), PHI,Rm(:,:,ik));
        end;
        Fm(:,:,ik)=Fi;
        Fm_li(:,:,ik)=Fi_li;
        Fm_ls(:,:,ik)=Fi_ls;
        Sem(:,:,ik)=Se2;
        relim(:,:,ik)=reli2;
            
        tmp=sprintf('Factor scores matrix %.0f done!',ik);disp(tmp);
        if corr==3
            if ik==k
                close(bar0);
            else
                BarCount = BarCount + 1; 
                compT = toc(TicPrint);
                compT = compT*(TotalLines-BarCount)/BarCount;
                s_tmp={};
                s_tmp{1}=sprintf('Computing factor scores ... Actual imputation =  %.0f  / %.0f ) ',ik+1,k);
                s_tmp{2}=sprintf('(Remaining time =  %s )',Time2String( compT ));
                waitbar(BarCount/TotalLines,bar0,s_tmp);
            end;
        end;
    end;

    for ik=1:k,
        F = F + Fm(:,:,ik);
        F_li= F_li + Fm_li(:,:,ik);
        F_ls= F_ls + Fm_ls(:,:,ik);
        Se= Se + Sem(:,:,ik);
        reli= reli + relim(:,:,ik);
    end;

    F = F / k;
    F = F*10+50;
    
    F_li = F_li / k;
    F_li = F_li*10+50;
    
    F_ls = F_ls / k;
    F_ls = F_ls*10+50;
    
    Se = Se / k;
    reli = reli / k;

    Fint=zeros(N,2,r+checking_DS+checking_AQ);
    for i=1:r+checking_DS+checking_AQ,
        tmp=[];
        for ik=1:k,
            tmp=[tmp Fm(:,i,ik)];
        end;
        Fint(:,1,i)=min(tmp')';
        Fint(:,2,i)=max(tmp')';
    end;
else
    W_enordre=[];
end;

