%% PARALLELPCA : PCA  Parallel Analysis (Monte Carlo technique)
% An extension of Horn's method (1965). It uses permutation technique to obtain random samples.
%
% ARGUMENTS   : R        -> Observed variance/covariance matrix
%               cases    -> Size of the sample related to R
%               Ndatsets -> Number of random datasets used to compute the random distribution of eigenvalues
%               percent  -> Desired percentile of distribution of random eigenvalues (for example 95 for the 95th percentile) to be used as threshold
%               type     -> Switch between dispersion matrix   
%                               1: Computes Pearson correlation matrices
%                               2: Computes Polychoric correlation matrices
%               display  -> Switch if the results will be displayed   
%                               0: Turns off the screen display of results
%                               1: Turns on the display
%                               2: Turns on the Printing function in an Output listbox
%               Output   -> Optional input, if it's present, the function will print in
%                           that argument. If isn't, the function will display the results in matlab
%                           window, using disp
%               Output2  -> Another optional input, if it's present, the function will be 
%                           printed in LaTeX code in that argument.
%               Output3  -> Another optional input, if it's present, the function will be
%                           printed in rtf code in that argument.
%
% RETURN VALUE: r       -> number of dimensions advised
%               dim     -> number of dimensions
%               realPA  -> vector of real eigenvalues
%               meansPA -> vector of mean random eigenvalues
%
% DEPENDENCIES: ed.m
%               MyPrint.m (if necessary)
%               Print_RTF.m (if necessary)
%               Time2String.m
%               nrm.m
%
% Last update: Urbano Lorenzo-Seva 21/04/2015
%

function [r,dim,realPA,meansPA] = parallelPCA(R,cases,Ndatsets, percent,display,Output,Output2,Output3)

tic;

if display == 1 || display == 2,
   tstart = tic;
   ch=0;
   total=Ndatsets+1;
   tmp3 = 0; bar1 = waitbar(tmp3,'Parallel Analysis');
end;    

m = size(R,1);  % number of variables.

[V,L] = ed(R);
realevals = diag(L);

evals=[];
for a = 1:Ndatsets,
    
   Z=randn(cases,m);
   [V,L]= ed(corrcoef(Z));
   L = diag(L);

   evals=[evals L];
   
   if (display >0),
        ch=ch+1;
        compT = toc(tstart);
        compT = compT*(total-ch)/ch;
        s_tmp=['Parallel Analysis (Remaining time = ' Time2String( compT ) ')'];
        waitbar(ch/total,bar1,s_tmp);
   end;
   
end;

means = (mean(evals,2));   % mean eigenvalues for each position.
evals = sort(evals,2);     % sorting the eigenvalues for each position.
percentiles = (evals(:,round((percent*Ndatsets)/100)));  % percentiles.
	
clear evals;
if display == 1 || display == 2
    close(bar1);
end;

if display==1,
    disp('--------------------------------------------------------------------------------');
    disp(' ');
    disp('PARALLEL ANALYSIS (PA) '); 
    disp('(Timmerman & Lorenzo-Seva, 2011)'); disp(' ');
    disp('Implementation details:'); disp(' ');
    disp('       Correlation matrices analized:                Pearson correlation matrices'); 
    disp(['       Number of random correlation matrices:        '  num2str(Ndatsets)]);
    disp('       Method to obtain random correlation matrices: Monte carlo random sampling method'); 
    disp(' ');disp(' ');
    disp(['Factor    Real-data        Mean of random   ' num2str(percent) ' percentile of random']);
    disp( '          % of variance    % of variance    % of variance');
    disp(' '); 
    nf_per=0; nf_mean=0;campana=0;
    for i=1:m,
        buff=sprintf(' %3.0f      ',i);
        if campana == 0,
            if realevals(i) > percentiles(i),
               buff2=sprintf('%5.2f**          %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
               nf_per=nf_per+1; nf_mean=nf_mean+1;
            elseif realevals(i) > means(i),
               buff2=sprintf('%5.2f*           %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
               nf_mean=nf_mean+1;
            else,
               buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
               campana=1;
            end;
        else,
            buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
        end;
        disp([buff buff2]);
    end;
    disp(' ');
    if nf_mean > nf_per,
       buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
       disp(buff);
       buff=sprintf('*   Advised number of factors:    %.0f',nf_mean);
       disp(buff);
    else
       buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
       disp(buff);
    end;
    disp(' ');
    %disp(['Time for this problem = ', num2str(toc) ]); disp([' '])
    disp(' ');
    
elseif display==2,
    if nargin==6
        MyPrint(Output,' ');
        MyPrint(Output,'PARALLEL ANALYSIS (PA) '); 
        MyPrint(Output,'(Timmerman & Lorenzo-Seva, 2011)');
        MyPrint(Output,' ');
        MyPrint(Output,'Implementation details:');
        MyPrint(Output,' ');
        MyPrint(Output,'       Correlation matrices analized:                User defined matrices'); 
        a=num2str(Ndatsets);
        buff = sprintf('       Number of random correlation matrices:       %s',a); 
        MyPrint(Output, buff);
        MyPrint(Output,'       Method to obtain random correlation matrices: Monte carlo random sampling method');
        MyPrint(Output,' ');
        MyPrint(Output,' ');
        b=num2str(percent);
        buff = sprintf('Factor    Real-data        Mean of random   %s',b);
        buff2 = sprintf('percentile of random');
        MyPrint(Output,[buff buff2]);
        MyPrint(Output, '          % of variance    % of variance    % of variance');
        MyPrint(Output,' ');

        nf_per=0; nf_mean=0;campana=0;
        for i=1:m,
            buff=sprintf(' %3.0f      ',i);
            if campana == 0,
                if realevals(i) > percentiles(i),
                   buff2=sprintf('%5.2f**          %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                   nf_per=nf_per+1; nf_mean=nf_mean+1;
                elseif realevals(i) > means(i),
                   buff2=sprintf('%5.2f*           %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                   nf_mean=nf_mean+1;
                else,
                   buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                   campana=1;
                end;
            else,
                buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
            end;
            MyPrint(Output,[buff buff2]);
        end;
        MyPrint(Output,' ')
        if nf_mean > nf_per,
           buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
           MyPrint(Output,buff);
           buff=sprintf('*   Advised number of factors:    %.0f',nf_mean);
           MyPrint(Output,buff);
        else
           buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
           MyPrint(Output,buff);
        end;
        MyPrint(Output,' ')
        %disp(['Time for this problem = ', num2str(toc) ]); disp([' '])
        MyPrint(Output,'-----------------------------------------------------------------------');
        MyPrint(Output,' ')
    
    elseif nargin==7,
        MyPrint(Output,' ');
        MyPrint(Output,'PARALLEL ANALYSIS (PA) '); 
        MyPrint(Output,'(Timmerman & Lorenzo-Seva, 2011)');
        MyPrint(Output,' ');
        MyPrint(Output,'Implementation details:');
        MyPrint(Output,' ');
        MyPrint(Output,'       Correlation matrices analized:                User defined matrices'); 
        a=num2str(Ndatsets);
        buff = sprintf('       Number of random correlation matrices:       %s',a); 
        MyPrint(Output, buff);
        MyPrint(Output,'       Method to obtain random correlation matrices: Monte carlo random sampling method');
        MyPrint(Output,' ');
        MyPrint(Output,' ');

        %%%%%%
        MyPrint(Output2,'\section{Parallel Analysis(PA)\normalsize(Horn, 1965)}');
        MyPrint(Output2,'\begin{table}[!htb]');
        MyPrint(Output2,'\caption{PA Implemention Details}');
        MyPrint(Output2,'\label{taula:PAImplemetion}');
        MyPrint(Output2,'\centering');
        MyPrint(Output2,'\begin{tabular}{l l} \hline');
        MyPrint(Output2,'Correlation matrices analized: & User defined matrices \\'); 
        buff = sprintf('Number of random correlation matrices: & %s',a); 
        buff9='\\';
        MyPrint(Output2, [buff buff9]);
        MyPrint(Output2,' Method to obtain random correlation matrices: & Monte carlo random sampling method \\');
        MyPrint(Output2,'\hline');
        MyPrint(Output2,'\end{tabular}');
        MyPrint(Output2,'\end{table}');
        %%%%%    
        b=num2str(percent);
        buff = sprintf('Factor    Real-data        Mean of random   %s',b);
        buff2 = sprintf('percentile of random');
        MyPrint(Output,[buff buff2]);
        MyPrint(Output, '          % of variance    % of variance    % of variance');
        MyPrint(Output,' ');
        MyPrint(Output2,'\begin{table}[!htb]');
        MyPrint(Output2,'\caption{Parallel Analyis Output}');
        MyPrint(Output2,'\label{taula:PA}');
        MyPrint(Output2,'\centering');
        MyPrint(Output2,'\begin{tabular}{l c c c}');
        perc='\%';
        MyPrint(Output2,'\hline');
        buff = sprintf('Factor & Real-data %s of variance & Mean of random %s of variance & %s percentile of random %s of variance',perc,perc,b,perc);
        MyPrint(Output2,[buff buff9]);
        MyPrint(Output2,'\hline');

        nf_per=0; nf_mean=0;campana=0;
        for i=1:m,
            buff=sprintf(' %3.0f      ',i);
            if campana == 0,
                if realevals(i) > percentiles(i),
                   buff2=sprintf('%5.2f**          %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                   lat2=sprintf('& %5.2f** & %5.2f  & %5.2f',realevals(i), means(i), percentiles(i));
                   nf_per=nf_per+1; nf_mean=nf_mean+1;
                elseif realevals(i) > means(i),
                   buff2=sprintf('%5.2f*           %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                   lat2=sprintf('& %5.2f*& %5.2f & %5.2f',realevals(i), means(i), percentiles(i));
                   nf_mean=nf_mean+1;
                else,
                   buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                   lat2=sprintf('& %5.2f & %5.2f & %5.2f',realevals(i), means(i), percentiles(i));
                   campana=1;
                end;
            else,
                buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                lat2=sprintf('& %5.2f & %5.2f & %5.2f',realevals(i), means(i), percentiles(i));
            end;
            MyPrint(Output,[buff buff2]);
            lat3 = '\\';
            MyPrint(Output2,[buff lat2 lat3]);
        end;
        MyPrint(Output2,'\hline');
        MyPrint(Output2,'\end{tabular}');
        MyPrint(Output,' ')
        if nf_mean > nf_per,
           buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
           MyPrint(Output,buff);
           MyPrint(Output2,buff);
           buff=sprintf('*   Advised number of factors:    %.0f',nf_mean);
           MyPrint(Output,buff);
           MyPrint(Output2,buff);
        else
           buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
           MyPrint(Output,buff);
           MyPrint(Output2,buff);
        end;
        MyPrint(Output,' ')
        %disp(['Time for this problem = ', num2str(toc) ]); disp([' '])
        MyPrint(Output,'-----------------------------------------------------------------------');
        MyPrint(Output,' ')
        MyPrint(Output2,'\end{table}');
        
    elseif nargin==8,
        %%% TEXT
        MyPrint(Output,' ');
        MyPrint(Output,'PARALLEL ANALYSIS (PA) '); 
        MyPrint(Output,'(Timmerman & Lorenzo-Seva, 2011)');
        MyPrint(Output,' ');
        MyPrint(Output,'Implementation details:');
        MyPrint(Output,' ');
        MyPrint(Output,'       Correlation matrices analized               : User defined matrices'); 
        a=num2str(Ndatsets);
        buff = sprintf('       Number of random correlation matrices       : %s',a); 
        MyPrint(Output, buff);
        MyPrint(Output,'       Method to obtain random correlation matrices: Monte carlo random sampling method');
        MyPrint(Output,' ');
        MyPrint(Output,' ');

        %%%%%% LATEX
        MyPrint(Output2,'\section{Parallel Analysis(PA)\normalsize(Horn, 1965)}');
        MyPrint(Output2,'\begin{table}[!htb]');
        MyPrint(Output2,'\caption{PA Implemention Details}');
        MyPrint(Output2,'\label{taula:PAImplemetion}');
        MyPrint(Output2,'\centering');
        MyPrint(Output2,'\begin{tabular}{l l} \toprule');
        MyPrint(Output2,'Correlation matrices analized: & User defined matrices \\'); 
        buff = sprintf('Number of random correlation matrices: & %s',a); 
        buff9='\\';
        MyPrint(Output2, [buff buff9]);
        MyPrint(Output2,' Method to obtain random correlation matrices: & Monte carlo random sampling method \\');
        MyPrint(Output2,'\bottomrule');
        MyPrint(Output2,'\end{tabular}');
        MyPrint(Output2,'\end{table}');
        
        %%%%% RTF    
        
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\qc \ul Parallel Analysis (PA) (Timmerman & Lorenzo-Seva, 2011)\ul0');
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\par');
        interline1='\trowd\trgaph144\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx10000';
        interline2='\trowd\trgaph144\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx5000\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx10000';
        MyPrint(Output3,interline1);
        int='\intbl ';
        cell='\cell';
        row='\row';
        buff='Implementation details \cell \row';
        MyPrint(Output3,[int buff]);
        MyPrint(Output3,'\ql');
        MyPrint(Output3,interline2);
        MyPrint(Output3,'\intbl Correlation matrices analized: \cell User defined matrices \cell \row'); 
        MyPrint(Output3,interline2);
        buff_rtf=sprintf('Number of random correlation matrices: %s %s %s %s',cell,a,cell,row);
        MyPrint(Output3,[int,buff_rtf]);
        MyPrint(Output3,interline2);
        buff_rtf='\intbl Method to obtain random correlation matrices: \cell Monte carlo random sampling method \cell \row \pard \par';
        MyPrint(Output3,buff_rtf);
        MyPrint(Output3,'\par');
        
        b=num2str(percent);
        buff = sprintf('Factor    Real-data        Mean of random   %s',b);
        buff2 = sprintf('percentile of random');
        MyPrint(Output,[buff buff2]);
        MyPrint(Output, '          % of variance    % of variance    % of variance');
        MyPrint(Output,' ');
        
        MyPrint(Output2,'\begin{center}');
        MyPrint(Output2,'\begin{longtable}[H]{l S S S}');
        MyPrint(Output2,'\caption{Parallel Analyis Output}');
        MyPrint(Output2,'\toprule');
        perc='\%';
        buff = '\multicolumn{1}{l}{\textbf{Factor}} & ';MyPrint(Output2,buff);
        buff = '\multicolumn{1}{p{3.5cm}}{\textbf';
        buff1 = sprintf('{Real-data %s of variance}} & ',perc);MyPrint(Output2,[buff buff1]);
        buff1 = sprintf('{Mean of random %s of variance}} & ',perc);MyPrint(Output2,[buff buff1]);
        buff1 = sprintf('{%s percentile of random %s of variance}} ',b,perc);MyPrint(Output2,[buff buff1 buff9]);
        MyPrint(Output2,'\midrule');
        MyPrint(Output2,'\endfirsthead');
        MyPrint(Output2,'\multicolumn{4}{c}%');
        MyPrint(Output2,'{\tablename\ \thetable\ -- \textit{(Continued from previous page)}} \\');
        MyPrint(Output2,'\toprule');
        buff = '\multicolumn{1}{l}{\textbf{Factor}} & ';MyPrint(Output2,buff);
        buff = '\multicolumn{1}{p{3.5cm}}{\textbf';
        buff1 = sprintf('{Real-data %s of variance}} & ',perc);MyPrint(Output2,[buff buff1]);
        buff1 = sprintf('{Mean of random %s of variance}} & ',perc);MyPrint(Output2,[buff buff1]);
        buff1 = sprintf('{%s percentile of random %s of variance}} ',b,perc);MyPrint(Output2,[buff buff1 buff9]);
        MyPrint(Output2,'\midrule');
        MyPrint(Output2,'\endhead');
        MyPrint(Output2,'\bottomrule \multicolumn{4}{r}{\textit{Continued on next page}} \\');
        MyPrint(Output2,'\endfoot');
        MyPrint(Output2,'\bottomrule');
        MyPrint(Output2,'\endlastfoot');
        
        %%%% 2nd table TEXT AND LATEX %%%
        nf_per=0; nf_mean=0;campana=0;
        for i=1:m,
            buff=sprintf(' %3.0f      ',i);
            if campana == 0,
                if realevals(i) > percentiles(i),
                   buff2=sprintf('%5.2f**          %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                   lat2=sprintf('& %5.2f** & %5.2f  & %5.2f',realevals(i), means(i), percentiles(i));
                   nf_per=nf_per+1; nf_mean=nf_mean+1;
                elseif realevals(i) > means(i),
                   buff2=sprintf('%5.2f*           %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                   lat2=sprintf('& %5.2f*& %5.2f & %5.2f',realevals(i), means(i), percentiles(i));
                   nf_mean=nf_mean+1;
                else,
                   buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                   lat2=sprintf('& %5.2f & %5.2f & %5.2f',realevals(i), means(i), percentiles(i));
                   campana=1;
                end;
            else,
                buff2=sprintf('%5.2f            %5.2f            %5.2f',realevals(i), means(i), percentiles(i));
                lat2=sprintf('& %5.2f & %5.2f & %5.2f',realevals(i), means(i), percentiles(i));
            end;
            MyPrint(Output,[buff buff2]);
            lat3 = '\\';
            MyPrint(Output2,[buff lat2 lat3]);
        end;
        MyPrint(Output,' ');
        MyPrint(Output2,'\end{longtable}');
        MyPrint(Output2,'\end{center}'); 
        
        if nf_mean > nf_per,
           buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
           MyPrint(Output,buff);
           MyPrint(Output2,[buff lat3]);
           buff=sprintf('*   Advised number of factors:    %.0f',nf_mean);
           MyPrint(Output,buff);
           MyPrint(Output2,[buff lat3]);
        else
           buff=sprintf('**  Advised number of factors:    %.0f',nf_per);
           MyPrint(Output,buff);
           MyPrint(Output2,[buff lat3]);
        end;
        MyPrint(Output,' ')
        %disp(['Time for this problem = ', num2str(toc) ]); disp([' '])
        MyPrint(Output,'-----------------------------------------------------------------------');
         
        
        %%%% RTF %%%%
        
        X=[realevals means percentiles];
        perc_rtf='%';
        buff={};
        buff{1} = sprintf('Factor');
        buff{2} = sprintf('Real-data %s of variance',perc_rtf);
        buff{3} = sprintf('Mean of random %s of variance',perc_rtf);
        buff{4} = sprintf('%s percentile of random %s of variance',b,perc_rtf);
        header = buff;
        
        buffs={};
        for i=1:m
            buff = sprintf('%3.0f',i);
            buffs(i)=cellstr(buff);
        end;
            
        Print_RTF(X,Output3,header,buffs); %fixed, no labels
        
        if nf_mean > nf_per,
           buff=sprintf('Advised number of factors: between %.0f and  %.0f',nf_per,nf_mean);
           MyPrint(Output3,buff);
           MyPrint(Output3,'\par');

        else
           buff=sprintf('Advised number of factors: %.0f',nf_per);
           MyPrint(Output3,buff);
           MyPrint(Output3,'\par');

        end;
        MyPrint(Output3,'\par')
    
    end;
end;
r = nf_per;
realPA = realevals;
meansPA = means;
dim = m;

return;